mod font;
mod panel;
mod ext;
mod title;
mod tray;
mod sensors;
mod util;
mod widget;
mod x11;

use chan;
use chan_signal;
use config::Config;

use std::thread;
use std::time;

pub const SIZE: u16 = 20;

pub fn ui_main(signal: chan::Receiver<chan_signal::Signal>, cfg: &Config) -> i32 {
    if let Some(conn) = x11::Connection::new() {
        let (tx, rx) = chan::sync(10);
        {
            let tx = tx.clone();
            let conn = conn.clone_connection();
            thread::spawn(move || {
                loop {
                    match conn.wait_for_event() {
                        Some(event) => {
                            let message = widget::WidgetMessage::XcbEvent(event);
                            tx.send(message);
                        },
                        None => { break; }
                    }
                }
            });
        }

        let mut panel = panel::Panel::new(conn, cfg);

        let title = title::Title::new(panel.conn.clone(), panel.make_draw_context());
        panel.add_left_widget(Box::new(title));

        let tray = tray::Tray::new(tx.clone(), panel.conn.clone(), panel.window);
        panel.add_right_widget(Box::new(tray));

        let sensors = sensors::Sensors::new(panel.make_draw_context(), cfg);
        panel.add_right_widget(Box::new(sensors));

        panel.create();

        {
            let tx = tx.clone();
            thread::spawn(move || {
                loop {
                    tx.send(widget::WidgetMessage::Update);
                    thread::sleep(time::Duration::from_secs(1));
                }
            });
        }

        loop {
            chan_select!(
                rx.recv() -> event => {
                    if panel.handle_event(event.unwrap()) {
                        println!("Exiting");
                        break;
                    }
                },
                signal.recv() => {
                    panel.finish();
                }
            );
        }
    }
    0
}
