use ui::widget;
use ui::widget::Widget;
use ui::x11;
use xcb;

use std::sync::Arc;

pub struct Title {
    conn: Arc<x11::Connection>,
    context: widget::DrawContext,
    title: String,
    last_pos: u16,
    last_width: u16,
    last_win: xcb::Window
}

impl Title {
    pub fn new(conn: Arc<x11::Connection>, context: widget::DrawContext) -> Title {
        Title {
            conn: conn,
            context: context,
            title: "".to_string(),
            last_pos: 0,
            last_width: 0,
            last_win: 0
        }
    }

    pub fn redraw(&mut self) {
        self.context.draw_bg(self.last_pos, self.last_width);
        self.context.draw_text(&self.title, self.last_pos);
        self.last_width = self.context.measure_text(&self.title);
        self.conn.flush();
    }
}

impl Widget for Title {
    fn init(&mut self) {
        let screen = self.conn.default_screen();
        self.last_win = screen.get_active_window();
        self.title = self.conn.get_window_name(self.last_win);
        self.conn.watch(screen.root, true);
        self.conn.watch(self.last_win, true);
    }

    fn render(&mut self, x: u16) {
        self.last_pos = x;
        self.context.draw_text(&self.title, self.last_pos);
        self.last_width = self.context.measure_text(&self.title);
    }

    fn width(&mut self) -> u16 {
        0
    }

    fn handle_event(&mut self, event: &widget::WidgetMessage) -> bool {
        match event {
            &widget::WidgetMessage::XcbEvent(ref event) =>
                if event.response_type() == xcb::PROPERTY_NOTIFY {
                    let event: &xcb::PropertyNotifyEvent = unsafe { xcb::cast_event(&event) };
                    if event.atom() == self.conn.atom(x11::_NET_ACTIVE_WINDOW) {
                        let new_win = {
                            let screen = self.conn.default_screen();
                            screen.get_active_window()
                        };
                        self.conn.watch(self.last_win, false);
                        self.conn.watch(new_win, true);
                        self.last_win = new_win;
                        self.title = self.conn.get_window_name(new_win);
                        self.redraw();
                    }
                    else if event.atom() == self.conn.atom(x11::_NET_WM_NAME) {
                        self.title = self.conn.get_window_name(self.last_win);
                        self.redraw();
                    }
                },
            _ => {}
        }
        false
    }

    fn finish(&mut self) {
        let screen = self.conn.default_screen();
        self.conn.watch(screen.root, false);
        self.conn.watch(self.last_win, false);
    }
}
